(document:surround "/std/frame")

;;; Functions

(define (ui-reset-password)
  (form-update-value-list
    '("passwd_1" "passwd_2")
    '("/users" 'a "b")) ;;empty command
  (form-update-value "auto" #f)
  (ui-auto #f)
  (ui-generate))


(define (ui-read . name)
  (let ((name (if (pair? name) (car name) (form-value "name"))))
    (ui-reset-password)
    (form-update-value-list
      '("gecos" "home" "shell" "allow_su")
      (if name (woo-read-first "/users" 'name name) '("/users")))))

(define (ui-list . name)
  (let* ((data (woo-list "/users"))
	 (name (or (and (pair? name) (car name))
		   (and (pair? data) (woo-get-option (car data) 'name)))))
    (form-update-enum "name" data)
    (and name (form-update-value "name" name))
    (ui-read name)))

(define (ui-auto . auto)
  (let ((auto (if (pair? auto) (car auto) (form-value "auto"))))
    (form-update-visibility
      '("passwd_auto" "generate_button")
      auto)
    (form-update-visibility
      '("passwd_1" "passwd_2")
      (not auto))))

(define (ui-generate)
  (form-update-value-list
    '("passwd_auto")
    (car (woo "generate" "/users"))))

;;FIXME: need common form-confirm feature
(define (ui-delete)
  (let ((name (form-value "name")))
    (if (not-empty-string? name)
      (if (eq? 'yes (document:popup-warning
		      (string-append (_ "Do you really want to delete user ")
				     name
				     " ?")
		      'no
		      'yes))
	(catch/message
	  (lambda()
	    (woo-delete "/users/" 'name name 'language (form-value "language"))
	    (ui-list)))))))

(define (ui-new)
  (let ((new_name (form-value "new_name")))
    (if (not-empty-string? new_name)
      (catch/message
	(lambda()
	  (woo-new "/users" 'new_name new_name 'language (form-value "language"))
	  (form-update-value "new_name" "")
	  (ui-list new_name))))))

(define (ui-write)
  (let ((name (form-value "name")))
    (if (not-empty-string? name)
      (catch/message
	(lambda()
	  (apply woo-write  "/users"
		 (form-value-list
		   '("name" "home" "gecos" "shell" "allow_su"
		     "passwd_1" "passwd_2" "passwd_auto" "auto")))
	  (ui-read name))))))

(define (ui-init)
  (form-update-enum "shell" (woo-list "/users/avail_shell"))
  (ui-list)

  (form-bind "name" "change" ui-read)
  (form-bind "auto" "change" ui-auto)
  (form-bind "generate_button" "click" ui-generate)

  (form-bind "delete_button" "click" ui-delete)
  (form-bind "apply_button" "click" ui-write)
  (form-bind "new_button" "click" ui-new))

;;; UI
(gridbox
  columns "100"
  align "top"
  margin 10

  (label)
  (hbox align "left"
	(label text (_ "New account:") nameref "new_name")
	(edit name "new_name")
	(button text (_ "Create") name "new_button"))

  (separator)
  (splitbox
    columns "30;70"
    (listbox name "name" align "top")
    (gridbox
      columns "0;100"
      ;;
      (label text (_ "Comment:") align "right" nameref "gecos")
      (edit name "gecos")
      ;;
      (label text (_ "Home directory:") align "right" nameref "home")
      (edit name "home")
      ;;
      (label text (_ "Shell:") align "right" nameref "shell")
      (combobox name "shell")
      ;;
      (spacer)
      (checkbox text (_ "Member of administrators group") name "allow_su")

      ;;
      (label colspan 2)

      ;;
      (label text (_ "Password:") align "top;right")
      (gridbox
	columns "100;0"
	(checkbox text (_ "Generate automatically") name "auto")
	(spacer)
	(edit name "passwd_1" echo "stars")
	(label nameref "passwd_1" text (small (_ "(enter passphrase)")))
	(edit name "passwd_2" echo "stars")
	(label nameref "passwd_2" text (small (_ "(repeat passphrase)")))
	(edit name "passwd_auto" visibility #f alterability #f)
	(button text (_ "Generate") name "generate_button" visibility #f)
	(label colspan 2 nameref "passwd_auto" visibility #f))

      ;;
      (label colspan 2)

      ;;
      (spacer)
      (hbox align "left"
	    (button text (_ "Apply") name "apply_button")
	    (label)
	    (button text (_ "Delete user") name "delete_button")))))

;;; Logic

(document:root (when loaded (ui-init)))

